@@ -31,6 +31,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/poll.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -126,6 +127,72 @@ intel_output_dpms(xf86OutputPtr output, int mode);
 static void
 intel_output_dpms_backlight(xf86OutputPtr output, int oldmode, int mode);
 
+static inline int
+crtc_id(struct intel_crtc *crtc)
+{
+	return crtc->mode_crtc->crtc_id;
+}
+
+#ifdef __OpenBSD__
+
+#include <dev/wscons/wsconsio.h>
+#include "xf86Priv.h"
+
+static void
+intel_output_backlight_set(xf86OutputPtr output, int level)
+{
+	struct intel_output *intel_output = output->driver_private;
+	struct wsdisplay_param param;
+
+	if (level > intel_output->backlight_max)
+		level = intel_output->backlight_max;
+	if (! intel_output->backlight_iface || level < 0)
+		return;
+
+	param.param = WSDISPLAYIO_PARAM_BRIGHTNESS;
+	param.curval = level;
+	if (ioctl(xf86Info.consoleFd, WSDISPLAYIO_SETPARAM, &param) == -1) {
+		xf86DrvMsg(output->scrn->scrnIndex, X_ERROR,
+			   "Failed to set backlight level: %s\n",
+			   strerror(errno));
+	}
+}
+
+static int
+intel_output_backlight_get(xf86OutputPtr output)
+{
+	struct wsdisplay_param param;
+
+	param.param = WSDISPLAYIO_PARAM_BRIGHTNESS;
+	if (ioctl(xf86Info.consoleFd, WSDISPLAYIO_GETPARAM, &param) == -1) {
+		xf86DrvMsg(output->scrn->scrnIndex, X_ERROR,
+			   "Failed to get backlight level: %s\n",
+			   strerror(errno));
+		return -1;
+	}
+
+	return param.curval;
+}
+
+static void
+intel_output_backlight_init(xf86OutputPtr output)
+{
+	struct intel_output *intel_output = output->driver_private;
+	struct wsdisplay_param param;
+
+	param.param = WSDISPLAYIO_PARAM_BRIGHTNESS;
+	if (ioctl(xf86Info.consoleFd, WSDISPLAYIO_GETPARAM, &param) == -1) {
+		intel_output->backlight_iface = NULL;
+		return;
+	}
+
+	intel_output->backlight_iface = "wscons";
+	intel_output->backlight_max = param.max;
+	intel_output->backlight_active_level = param.curval;
+}
+
+#else
+
 #define BACKLIGHT_CLASS "/sys/class/backlight"
 
 /*
@@ -154,12 +221,6 @@ static const char *backlight_interfaces[] = {
 /* Enough for 10 digits of backlight + '\n' + '\0' */
 #define BACKLIGHT_VALUE_LEN 12
 
-static inline int
-crtc_id(struct intel_crtc *crtc)
-{
-	return crtc->mode_crtc->crtc_id;
-}
-
 static void
 intel_output_backlight_set(xf86OutputPtr output, int level)
 {
@@ -270,6 +331,7 @@ intel_output_backlight_init(xf86OutputPtr output)
 			intel_output->backlight_iface = str;
 			intel_output->backlight_max = intel_output_backlight_get_max(output);
 			if (intel_output->backlight_max > 0) {
+				intel_output->backlight_active_level = intel_output_backlight_get(output);
 				xf86DrvMsg(output->scrn->scrnIndex, X_CONFIG,
 					   "found backlight control interface %s\n", path);
 				return;
@@ -295,6 +357,7 @@ intel_output_backlight_init(xf86OutputPtr output)
 	intel_output->backlight_iface = NULL;
 }
 
+#endif
 
 static void
 mode_from_kmode(ScrnInfoPtr scrn,
@@ -494,7 +557,7 @@ intel_crtc_set_mode_major(xf86CrtcPtr crtc, DisplayModePtr mode,
 			return FALSE;
 		}
 
-		drm_intel_gem_bo_disable_resuse(intel->front_buffer);
+		drm_intel_bo_disable_reuse(intel->front_buffer);
 	}
 
 	saved_mode = crtc->mode;
@@ -599,7 +662,7 @@ intel_crtc_shadow_allocate(xf86CrtcPtr crtc, int width, int height)
 		return NULL;
 	}
 
-	drm_intel_gem_bo_disable_resuse(intel_crtc->rotate_bo);
+	drm_intel_bo_disable_reuse(intel_crtc->rotate_bo);
 
 	intel_crtc->rotate_pitch = rotate_pitch;
 	return intel_crtc->rotate_bo;
@@ -727,7 +790,7 @@ intel_set_scanout_pixmap(xf86CrtcPtr crtc, PixmapPtr ppix)
 		ErrorF("have front buffer\n");
 	}
 
-	drm_intel_gem_bo_disable_resuse(bo);
+	drm_intel_bo_disable_reuse(bo);
 
 	intel_crtc->scanout_pixmap = ppix;
 	return drmModeAddFB(intel->drmSubFD, ppix->drawable.width,
@@ -1500,7 +1563,7 @@ intel_xf86crtc_resize(ScrnInfoPtr scrn, int width, int height)
 	if (ret)
 		goto fail;
 
-	drm_intel_gem_bo_disable_resuse(intel->front_buffer);
+	drm_intel_bo_disable_reuse(intel->front_buffer);
 	intel->front_pitch = pitch;
 	intel->front_tiling = tiling;
 
@@ -1562,7 +1625,7 @@ intel_do_pageflip(intel_screen_private *intel,
 			 new_front->handle, &new_fb_id))
 		goto error_out;
 
-	drm_intel_gem_bo_disable_resuse(new_front);
+	drm_intel_bo_disable_reuse(new_front);
 	intel_glamor_flush(intel);
 	intel_batch_submit(scrn);
 
@@ -1830,6 +1893,30 @@ intel_mode_remove_fb(intel_screen_private *intel)
 	}
 }
 
+static Bool has_pending_events(int fd)
+{
+	struct pollfd pfd;
+	pfd.fd = fd;
+	pfd.events = POLLIN;
+	return poll(&pfd, 1, 0) == 1;
+}
+
+void
+intel_mode_close(intel_screen_private *intel)
+{
+	struct intel_mode *mode = intel->modes;
+
+	if (mode == NULL)
+		return;
+
+	while (has_pending_events(mode->fd))
+		drmHandleEvent(mode->fd, &mode->event_context);
+
+	RemoveBlockAndWakeupHandlers((BlockHandlerProcPtr)NoopDDA,
+				     drm_wakeup_handler, mode);
+	RemoveGeneralSocket(mode->fd);
+}
+
 void
 intel_mode_fini(intel_screen_private *intel)
 {
@@ -2026,7 +2113,9 @@ void intel_copy_fb(ScrnInfoPtr scrn)
 				0, 0,
 				scrn->virtualX, scrn->virtualY);
 	intel->uxa_driver->done_copy(dst);
+#if ABI_VIDEODRV_VERSION >= SET_ABI_VERSION(10, 0)
 	pScreen->canDoBGNoneRoot = TRUE;
+#endif
 
 cleanup_dst:
 	(*pScreen->DestroyPixmap)(dst);
