@@ -1537,11 +1537,7 @@ prepare_blt_copy(struct sna *sna,
 
 	assert(op->dst.bo);
 	assert(kgem_bo_can_blt(&sna->kgem, op->dst.bo));
-
-	if (!kgem_bo_can_blt(&sna->kgem, bo)) {
-		DBG(("%s: fallback -- can't blt from source\n", __FUNCTION__));
-		return false;
-	}
+	assert(kgem_bo_can_blt(&sna->kgem, bo));
 
 	if (!kgem_check_many_bo_fenced(&sna->kgem, op->dst.bo, bo, NULL)) {
 		kgem_submit(&sna->kgem);
@@ -1919,16 +1915,8 @@ prepare_blt_put(struct sna *sna,
 		struct sna_composite_op *op,
 		uint32_t alpha_fixup)
 {
-	PixmapPtr src = op->u.blt.src_pixmap;
-
 	DBG(("%s\n", __FUNCTION__));
 
-	if (!sna_pixmap_move_to_cpu(src, MOVE_READ))
-		return false;
-
-	assert(src->devKind);
-	assert(src->devPrivate.ptr);
-
 	if (op->dst.bo) {
 		assert(op->dst.bo == sna_pixmap(op->dst.pixmap)->gpu_bo);
 		if (alpha_fixup) {
@@ -1958,15 +1946,6 @@ prepare_blt_put(struct sna *sna,
 	return true;
 }
 
-static bool source_is_gpu(PixmapPtr pixmap, const BoxRec *box)
-{
-	struct sna_pixmap *priv = sna_pixmap(pixmap);
-	if (priv == NULL)
-		return false;
-	return sna_damage_contains_box(priv->gpu_damage,
-				       box) != PIXMAN_REGION_OUT;
-}
-
 #define alphaless(format) PICT_FORMAT(PICT_FORMAT_BPP(format),		\
 				      PICT_FORMAT_TYPE(format),		\
 				      0,				\
@@ -1981,21 +1960,6 @@ is_clear(PixmapPtr pixmap)
 	return priv && priv->clear;
 }
 
-static struct kgem_bo *
-peek_bo(DrawablePtr draw)
-{
-	struct sna_pixmap *priv;
-
-	if (draw == NULL)
-		return NULL;
-
-	priv = sna_pixmap(get_drawable_pixmap(draw));
-	if (priv == NULL)
-		return NULL;
-
-	return priv->gpu_bo;
-}
-
 bool
 sna_blt_composite(struct sna *sna,
 		  uint32_t op,
@@ -2013,6 +1977,7 @@ sna_blt_composite(struct sna *sna,
 	int16_t tx, ty;
 	BoxRec dst_box, src_box;
 	uint32_t alpha_fixup;
+	uint32_t color, hint;
 	bool was_clear;
 	bool ret;
 
@@ -2045,28 +2010,33 @@ sna_blt_composite(struct sna *sna,
 	} else
 		sna_render_picture_extents(dst, &dst_box);
 
-	bo = sna_pixmap(tmp->dst.pixmap)->gpu_bo;
-	if (bo == NULL || bo != peek_bo(src->pDrawable))
-		bo = sna_drawable_use_bo(dst->pDrawable, PREFER_GPU,
-					 &dst_box, &tmp->damage);
-	if (bo && !kgem_bo_can_blt(&sna->kgem, bo)) {
-		DBG(("%s: can not blit to dst, tiling? %d, pitch? %d\n",
-		     __FUNCTION__, bo->tiling, bo->pitch));
-		return false;
-	}
-
 	tmp->dst.format = dst->format;
 	tmp->dst.width = tmp->dst.pixmap->drawable.width;
 	tmp->dst.height = tmp->dst.pixmap->drawable.height;
 	get_drawable_deltas(dst->pDrawable, tmp->dst.pixmap,
 			    &tmp->dst.x, &tmp->dst.y);
-	tmp->dst.bo = bo;
 
 	if (op == PictOpClear) {
 clear:
 		if (was_clear)
 			return prepare_blt_nop(sna, tmp);
 
+		hint = 0;
+		if (can_render(sna)) {
+			hint |= PREFER_GPU;
+			if (sna_pixmap(tmp->dst.pixmap)->gpu_bo)
+				hint |= FORCE_GPU;
+			if (dst->pCompositeClip->data == NULL)
+				hint |= IGNORE_CPU;
+		}
+		tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable, hint,
+						  &dst_box, &tmp->damage);
+		if (tmp->dst.bo && !kgem_bo_can_blt(&sna->kgem, tmp->dst.bo)) {
+			DBG(("%s: can not blit to dst, tiling? %d, pitch? %d\n",
+			     __FUNCTION__, tmp->dst.bo->tiling, tmp->dst.bo->pitch));
+			return false;
+		}
+
 		if (!tmp->dst.bo) {
 			RegionRec region;
 
@@ -2096,6 +2066,27 @@ clear:
 			return false;
 		}
 
+		color = get_solid_color(src, tmp->dst.format);
+fill:
+		if (color == 0)
+			goto clear;
+
+		hint = 0;
+		if (can_render(sna)) {
+			hint |= PREFER_GPU;
+			if (sna_pixmap(tmp->dst.pixmap)->gpu_bo)
+				hint |= FORCE_GPU;
+			if (dst->pCompositeClip->data == NULL)
+				hint |= IGNORE_CPU;
+		}
+		tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable, hint,
+						  &dst_box, &tmp->damage);
+		if (tmp->dst.bo && !kgem_bo_can_blt(&sna->kgem, tmp->dst.bo)) {
+			DBG(("%s: can not blit to dst, tiling? %d, pitch? %d\n",
+			     __FUNCTION__, tmp->dst.bo->tiling, tmp->dst.bo->pitch));
+			return false;
+		}
+
 		if (!tmp->dst.bo) {
 			RegionRec region;
 
@@ -2107,7 +2098,7 @@ clear:
 				return false;
 		}
 
-		return prepare_blt_fill(sna, tmp, get_solid_color(src, tmp->dst.format));
+		return prepare_blt_fill(sna, tmp, color);
 	}
 
 	if (!src->pDrawable) {
@@ -2141,7 +2132,7 @@ clear:
 
 	if ((x >= src->pDrawable->width ||
 	     y >= src->pDrawable->height ||
-	     x + width <= 0 ||
+	     x + width  <= 0 ||
 	     y + height <= 0) &&
 	    (!src->repeat || src->repeatType == RepeatNone)) {
 		DBG(("%s: source is outside of valid area, converting to clear\n",
@@ -2151,9 +2142,9 @@ clear:
 
 	src_pixmap = get_drawable_pixmap(src->pDrawable);
 	if (is_clear(src_pixmap)) {
-		return prepare_blt_fill(sna, tmp,
-					color_convert(sna_pixmap(src_pixmap)->clear_color,
-						      src->format, tmp->dst.format));
+		color = color_convert(sna_pixmap(src_pixmap)->clear_color,
+				      src->format, tmp->dst.format);
+		goto fill;
 	}
 
 	alpha_fixup = 0;
@@ -2170,7 +2161,7 @@ clear:
 
 	/* XXX tiling? fixup extend none? */
 	if (x < 0 || y < 0 ||
-	    x + width > src->pDrawable->width ||
+	    x + width  > src->pDrawable->width ||
 	    y + height > src->pDrawable->height) {
 		DBG(("%s: source extends outside (%d, %d), (%d, %d) of valid drawable %dx%d, repeat=%d\n",
 		     __FUNCTION__,
@@ -2208,61 +2199,88 @@ clear:
 	     __FUNCTION__,
 	     tmp->dst.x, tmp->dst.y, tmp->u.blt.sx, tmp->u.blt.sy, alpha_fixup));
 
-	ret = false;
 	src_box.x1 = x;
 	src_box.y1 = y;
 	src_box.x2 = x + width;
 	src_box.y2 = y + height;
-	bo = NULL;
-	if (tmp->dst.bo || source_is_gpu(src_pixmap, &src_box))
-		bo = __sna_render_pixmap_bo(sna, src_pixmap, &src_box, true);
-	if (bo) {
-		if (!tmp->dst.bo)
-			tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable,
-							  FORCE_GPU | PREFER_GPU,
-							  &dst_box,
-							  &tmp->damage);
+	bo = __sna_render_pixmap_bo(sna, src_pixmap, &src_box, true);
+	if (bo && !kgem_bo_can_blt(&sna->kgem, bo)) {
+		DBG(("%s: can not blit from src size=%dx%d, tiling? %d, pitch? %d\n",
+		     __FUNCTION__,
+		     src_pixmap->drawable.width  < sna->render.max_3d_size,
+		     src_pixmap->drawable.height < sna->render.max_3d_size,
+		     bo->tiling, bo->pitch));
 
+		if (src_pixmap->drawable.width  <= sna->render.max_3d_size &&
+		    src_pixmap->drawable.height <= sna->render.max_3d_size &&
+		    bo->pitch <= sna->render.max_3d_pitch)
+		{
+			return false;
+		}
+
+		bo = NULL;
+	}
+
+	hint = 0;
+	if (bo || can_render(sna)) {
+		hint |= PREFER_GPU;
+		if (dst->pCompositeClip->data == NULL)
+			hint |= IGNORE_CPU;
+		if (bo)
+			hint |= FORCE_GPU;
+	}
+	tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable, hint,
+					  &dst_box, &tmp->damage);
+
+	ret = false;
+	if (bo) {
 		if (!tmp->dst.bo) {
 			DBG(("%s: fallback -- unaccelerated read back\n",
 			     __FUNCTION__));
+			if (!kgem_bo_is_busy(bo))
+				goto put;
 		} else if (bo->snoop && tmp->dst.bo->snoop) {
 			DBG(("%s: fallback -- can not copy between snooped bo\n",
 			     __FUNCTION__));
+			goto put;
+		} else if (!kgem_bo_can_blt(&sna->kgem, tmp->dst.bo)) {
+			DBG(("%s: fallback -- unaccelerated upload\n",
+			     __FUNCTION__));
+			if (!kgem_bo_is_busy(tmp->dst.bo) &&
+			    !kgem_bo_is_busy(bo))
+				goto put;
 		} else {
 			ret = prepare_blt_copy(sna, tmp, bo, alpha_fixup);
 			if (fallback && !ret)
 				goto put;
 		}
 	} else {
+		RegionRec region;
+
 put:
-		if (tmp->dst.bo) {
-			struct sna_pixmap *priv = sna_pixmap(tmp->dst.pixmap);
-			if (tmp->dst.bo == priv->cpu_bo) {
-				assert(kgem_bo_is_busy(tmp->dst.bo));
-				tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable,
-								  FORCE_GPU | PREFER_GPU,
-								  &dst_box,
-								  &tmp->damage);
-				if (tmp->dst.bo == priv->cpu_bo) {
-					DBG(("%s: forcing the stall to overwrite a busy CPU bo\n", __FUNCTION__));
-					tmp->dst.bo = NULL;
-					tmp->damage = NULL;
-				}
-			}
+		if (tmp->dst.bo == sna_pixmap(tmp->dst.pixmap)->cpu_bo) {
+			tmp->dst.bo = NULL;
+			tmp->damage = NULL;
 		}
 
 		if (tmp->dst.bo == NULL) {
-			RegionRec region;
+			hint = MOVE_INPLACE_HINT | MOVE_WRITE;
+			if (dst->pCompositeClip->data)
+				hint |= MOVE_READ;
 
 			region.extents = dst_box;
 			region.data = NULL;
-
-			if (!sna_drawable_move_region_to_cpu(dst->pDrawable, &region,
-							     MOVE_INPLACE_HINT | MOVE_READ | MOVE_WRITE))
+			if (!sna_drawable_move_region_to_cpu(dst->pDrawable,
+							     &region, hint))
 				return false;
 		}
 
+		region.extents = src_box;
+		region.data = NULL;
+		if (!sna_drawable_move_region_to_cpu(&src_pixmap->drawable,
+						     &region, MOVE_READ))
+			return false;
+
 		ret = prepare_blt_put(sna, tmp, alpha_fixup);
 	}
 
